from .model_version import ModelVersion


class Model():
    '''

    '''

    def __init__(self, client, name=None, modelId=None, modelConfig=None, createdAt=None, projectId=None, shared=None, sharedAt=None, refreshSchedules=None, latestModelVersion={}):
        self.client = client
        self.id = modelId
        self.name = name
        self.model_id = modelId
        self.model_config = modelConfig
        self.created_at = createdAt
        self.project_id = projectId
        self.shared = shared
        self.shared_at = sharedAt
        self.refresh_schedules = refreshSchedules
        self.latest_model_version = client._build_class(
            ModelVersion, latestModelVersion)

    def __repr__(self):
        return f"Model(name={repr(self.name)}, model_id={repr(self.model_id)}, model_config={repr(self.model_config)}, created_at={repr(self.created_at)}, project_id={repr(self.project_id)}, shared={repr(self.shared)}, shared_at={repr(self.shared_at)}, refresh_schedules={repr(self.refresh_schedules)}, latest_model_version={repr(self.latest_model_version)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'name': self.name, 'model_id': self.model_id, 'model_config': self.model_config, 'created_at': self.created_at, 'project_id': self.project_id, 'shared': self.shared, 'shared_at': self.shared_at, 'refresh_schedules': self.refresh_schedules, 'latest_model_version': self.latest_model_version.to_dict() if self.latest_model_version else None}

    def refresh(self):
        self = self.describe()
        return self

    def describe(self):
        return self.client.describe_model(self.model_id)

    def get_metrics(self, model_version=None, baseline_metrics=False):
        return self.client.get_model_metrics(self.model_id, model_version, baseline_metrics)

    def list_versions(self):
        return self.client.list_model_versions(self.model_id)

    def retrain(self, deployment_ids=[]):
        return self.client.retrain_model(self.model_id, deployment_ids)

    def cancel_training(self):
        return self.client.cancel_model_training(self.model_id)

    def delete(self):
        return self.client.delete_model(self.model_id)

    def create_deployment(self, name=None, description=None, deployment_config=None):
        return self.client.create_deployment(self.model_id, name, description, deployment_config)

    def wait_for_training(self, timeout=None):
        return self.client._poll(self, {'PENDING', 'TRAINING'}, delay=30, timeout=timeout)

    def wait_for_evaluation(self, timeout=None):
        return self.wait_for_training()

    def get_status(self):
        return self.describe().latest_model_version.status





class DeploymentAuthToken():
    '''

    '''

    def __init__(self, client, deploymentToken=None, createdAt=None):
        self.client = client
        self.id = None
        self.deployment_token = deploymentToken
        self.created_at = createdAt

    def __repr__(self):
        return f"DeploymentAuthToken(deployment_token={repr(self.deployment_token)}, created_at={repr(self.created_at)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'deployment_token': self.deployment_token, 'created_at': self.created_at}



from .refresh_policy import RefreshPolicy


class RefreshPipelineRun():
    '''

    '''

    def __init__(self, client, refreshPipelineRunId=None, refreshPolicyId=None, createdAt=None, startedAt=None, completedAt=None, status=None, refreshType=None, datasetVersions=None, modelVersions=None, deploymentVersions=None, batchPredictions=None, refreshPolicy={}):
        self.client = client
        self.id = refreshPipelineRunId
        self.refresh_pipeline_run_id = refreshPipelineRunId
        self.refresh_policy_id = refreshPolicyId
        self.created_at = createdAt
        self.started_at = startedAt
        self.completed_at = completedAt
        self.status = status
        self.refresh_type = refreshType
        self.dataset_versions = datasetVersions
        self.model_versions = modelVersions
        self.deployment_versions = deploymentVersions
        self.batch_predictions = batchPredictions
        self.refresh_policy = client._build_class(RefreshPolicy, refreshPolicy)

    def __repr__(self):
        return f"RefreshPipelineRun(refresh_pipeline_run_id={repr(self.refresh_pipeline_run_id)}, refresh_policy_id={repr(self.refresh_policy_id)}, created_at={repr(self.created_at)}, started_at={repr(self.started_at)}, completed_at={repr(self.completed_at)}, status={repr(self.status)}, refresh_type={repr(self.refresh_type)}, dataset_versions={repr(self.dataset_versions)}, model_versions={repr(self.model_versions)}, deployment_versions={repr(self.deployment_versions)}, batch_predictions={repr(self.batch_predictions)}, refresh_policy={repr(self.refresh_policy)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'refresh_pipeline_run_id': self.refresh_pipeline_run_id, 'refresh_policy_id': self.refresh_policy_id, 'created_at': self.created_at, 'started_at': self.started_at, 'completed_at': self.completed_at, 'status': self.status, 'refresh_type': self.refresh_type, 'dataset_versions': self.dataset_versions, 'model_versions': self.model_versions, 'deployment_versions': self.deployment_versions, 'batch_predictions': self.batch_predictions, 'refresh_policy': self.refresh_policy.to_dict() if self.refresh_policy else None}





class ModelVersion():
    '''

    '''

    def __init__(self, client, modelVersion=None, status=None, modelId=None, trainingStartedAt=None, trainingCompletedAt=None, datasetVersions=None):
        self.client = client
        self.id = None
        self.model_version = modelVersion
        self.status = status
        self.model_id = modelId
        self.training_started_at = trainingStartedAt
        self.training_completed_at = trainingCompletedAt
        self.dataset_versions = datasetVersions

    def __repr__(self):
        return f"ModelVersion(model_version={repr(self.model_version)}, status={repr(self.status)}, model_id={repr(self.model_id)}, training_started_at={repr(self.training_started_at)}, training_completed_at={repr(self.training_completed_at)}, dataset_versions={repr(self.dataset_versions)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'model_version': self.model_version, 'status': self.status, 'model_id': self.model_id, 'training_started_at': self.training_started_at, 'training_completed_at': self.training_completed_at, 'dataset_versions': self.dataset_versions}





class ExternalConnection():
    '''

    '''

    def __init__(self, client, externalConnectionId=None, service=None, name=None, createdAt=None, status=None):
        self.client = client
        self.id = externalConnectionId
        self.external_connection_id = externalConnectionId
        self.service = service
        self.name = name
        self.created_at = createdAt
        self.status = status

    def __repr__(self):
        return f"ExternalConnection(external_connection_id={repr(self.external_connection_id)}, service={repr(self.service)}, name={repr(self.name)}, created_at={repr(self.created_at)}, status={repr(self.status)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'external_connection_id': self.external_connection_id, 'service': self.service, 'name': self.name, 'created_at': self.created_at, 'status': self.status}





class ModelMetrics():
    '''

    '''

    def __init__(self, client, modelId=None, modelVersion=None, metrics=None, baselineMetrics=None):
        self.client = client
        self.id = None
        self.model_id = modelId
        self.model_version = modelVersion
        self.metrics = metrics
        self.baseline_metrics = baselineMetrics

    def __repr__(self):
        return f"ModelMetrics(model_id={repr(self.model_id)}, model_version={repr(self.model_version)}, metrics={repr(self.metrics)}, baseline_metrics={repr(self.baseline_metrics)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'model_id': self.model_id, 'model_version': self.model_version, 'metrics': self.metrics, 'baseline_metrics': self.baseline_metrics}





class ModelUpload():
    '''

    '''

    def __init__(self, client, modelId=None, modelInstanceId=None, status=None, createdAt=None, modelUploadId=None, embeddingsUploadId=None, verificationsUploadId=None):
        self.client = client
        self.id = modelUploadId
        self.model_id = modelId
        self.model_instance_id = modelInstanceId
        self.status = status
        self.created_at = createdAt
        self.model_upload_id = modelUploadId
        self.embeddings_upload_id = embeddingsUploadId
        self.verifications_upload_id = verificationsUploadId

    def __repr__(self):
        return f"ModelUpload(model_id={repr(self.model_id)}, model_instance_id={repr(self.model_instance_id)}, status={repr(self.status)}, created_at={repr(self.created_at)}, model_upload_id={repr(self.model_upload_id)}, embeddings_upload_id={repr(self.embeddings_upload_id)}, verifications_upload_id={repr(self.verifications_upload_id)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'model_id': self.model_id, 'model_instance_id': self.model_instance_id, 'status': self.status, 'created_at': self.created_at, 'model_upload_id': self.model_upload_id, 'embeddings_upload_id': self.embeddings_upload_id, 'verifications_upload_id': self.verifications_upload_id}





class BucketVerificationInstructions():
    '''

    '''

    def __init__(self, client, verified=None, writePermission=None, authOptions=None):
        self.client = client
        self.id = None
        self.verified = verified
        self.write_permission = writePermission
        self.auth_options = authOptions

    def __repr__(self):
        return f"BucketVerificationInstructions(verified={repr(self.verified)}, write_permission={repr(self.write_permission)}, auth_options={repr(self.auth_options)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'verified': self.verified, 'write_permission': self.write_permission, 'auth_options': self.auth_options}





class UseCase():
    '''
        Use Case Description
    '''

    def __init__(self, client, useCase=None, prettyName=None, description=None):
        self.client = client
        self.id = None
        self.use_case = useCase
        self.pretty_name = prettyName
        self.description = description

    def __repr__(self):
        return f"UseCase(use_case={repr(self.use_case)}, pretty_name={repr(self.pretty_name)}, description={repr(self.description)})"

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id

    def to_dict(self):
        return {'use_case': self.use_case, 'pretty_name': self.pretty_name, 'description': self.description}
